from telegram._utils.defaultvalue import DEFAULT_80 ,DEFAULT_IP ,DEFAULT_NONE ,DEFAULT_TRUE ,DefaultValue
from telegram._utils.types import SCT, DVType, ODVInput
from typing import TYPE_CHECKING,Any,AsyncContextManager,Awaitable,Callable,Coroutine,DefaultDict,Dict,Generator,Generic,List,Mapping,NoReturn,Optional,Sequence,Set,Tuple,Type,TypeVar,Union

from telegram.ext import Application , ContextTypes , CallbackContext , filters , CommandHandler , MessageHandler , CallbackQueryHandler , InlineQueryHandler , MessageReactionHandler , ChatJoinRequestHandler

from ._handlers import HANDLERS , MessagesHandlers




LOGO = """
........................................
.#####...####...##...##...####...#####..
.##..##.##..##..###.###..##..##..##..##.
.#####..######..##.#.##..##..##..##..##.
.##.....##..##..##...##..##..##..##..##.
.##.....##..##..##...##...####...#####..
........................................    
   ├ ᴄᴏᴘʏʀɪɢʜᴛ © 𝟸𝟶𝟸𝟹-𝟸𝟶𝟸𝟺 ᴘᴀᴍᴏᴅ ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ. ᴀʟʟ ʀɪɢʜᴛs ʀᴇsᴇʀᴠᴇᴅ.
   ├ ʟɪᴄᴇɴsᴇᴅ ᴜɴᴅᴇʀ ᴛʜᴇ  ɢᴘʟ-𝟹.𝟶 ʟɪᴄᴇɴsᴇ.
   └ ʏᴏᴜ ᴍᴀʏ ɴᴏᴛ ᴜsᴇ ᴛʜɪs ғɪʟᴇ ᴇxᴄᴇᴘᴛ ɪɴ ᴄᴏᴍᴘʟɪᴀɴᴄᴇ ᴡɪᴛʜ ᴛʜᴇ ʟɪᴄᴇɴsᴇ.
"""

def built_in_error(update, context):
    print(f'Update {update} \n\nCaused error {context.error}')

class Client(Application):
    def __init__(
            self, 
            TOKEN: str, 
            PORT: DVType[int] = DEFAULT_80,
            WEBHOOK_URL:  Optional[str] = None,
            HANDLERS: HANDLERS = {},
    ):
    
        self.token = TOKEN
        self.port = PORT
        self.webhook_url = WEBHOOK_URL
        self.handlers = HANDLERS
        self.app = Application.builder().token(self.token).build()

        if self.handlers == {}:
            raise "No Handlers found , please set handlers to start bot"
        
        commands = (HANDLERS.commands).items()
        messages:MessagesHandlers = HANDLERS.messages
        callback = HANDLERS.callback
        inline = HANDLERS.inline
        join_request = HANDLERS.join_request
        reaction = HANDLERS.reaction
        error = HANDLERS.error


        for command , cmd_func in commands:
            self.app.add_handler(CommandHandler(command , cmd_func))

        if messages.TEXT:self.app.add_handler(MessageHandler(filters.TEXT & ~filters.COMMAND,messages.TEXT))
        if messages.POLL:self.app.add_handler(MessageHandler(filters.POLL,messages.POLL))
        if messages.REPLY:self.app.add_handler(MessageHandler(filters.REPLY,messages.REPLY))
        if messages.AUDIO:self.app.add_handler(MessageHandler(filters.AUDIO,messages.AUDIO))
        if messages.VIDEO:self.app.add_handler(MessageHandler(filters.VIDEO,messages.VIDEO))
        if messages.VOICE:self.app.add_handler(MessageHandler(filters.VOICE,messages.VOICE ))
        if messages.CAPTION:self.app.add_handler(MessageHandler(filters.CAPTION,messages.CAPTION))
        if messages.CONTACT:self.app.add_handler(MessageHandler(filters.CONTACT,messages.CONTACT))
        if messages.LOCATION:self.app.add_handler(MessageHandler(filters.LOCATION,messages.LOCATION))
        if messages.STICKER:self.app.add_handler(MessageHandler(filters.Sticker.ALL,messages.STICKER))
        if messages.DOCUMENT:self.app.add_handler(MessageHandler(filters.ATTACHMENT,messages.DOCUMENT))
        if messages.ALLSTATUS:self.app.add_handler(MessageHandler(filters.StatusUpdate.ALL,messages.ALLSTATUS))
        if messages.NEW_CHAT_PHOTO:self.app.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_PHOTO,messages.NEW_CHAT_PHOTO))
        if messages.NEW_CHAT_MEMBERS:self.app.add_handler(MessageHandler(filters.StatusUpdate.NEW_CHAT_MEMBERS,messages.NEW_CHAT_MEMBERS))
        if messages.LEFT_CHAT_MEMBER:self.app.add_handler(MessageHandler(filters.StatusUpdate.LEFT_CHAT_MEMBER,messages.LEFT_CHAT_MEMBER))
        if messages.PINNED_MESSAGE:self.app.add_handler(MessageHandler(filters.StatusUpdate.PINNED_MESSAGE,messages.PINNED_MESSAGE))
        if messages.MESSAGE_AUTO_DELETE_TIMER_CHANGED:self.app.add_handler(MessageHandler(filters.StatusUpdate.MESSAGE_AUTO_DELETE_TIMER_CHANGED,messages.MESSAGE_AUTO_DELETE_TIMER_CHANGED))

        if callback:
            self.app.add_handler(CallbackQueryHandler(callback))

        if inline:
            self.app.add_handler(InlineQueryHandler(inline))

        if join_request:
            self.app.add_handler(ChatJoinRequestHandler(join_request))

        if reaction:
            self.app.add_handler(MessageReactionHandler(reaction))

        if error:
            self.app.add_error_handler(error)
        else:
            self.app.add_error_handler(built_in_error)

        
    def start(
            self,
            drop_pending_updates:Optional[bool] = None,
            ):
        print(LOGO + "\n\n" )
        print("Bot Started")
        
        try:
            if self.webhook_url != None:
                print("running webhook...")
                self.app.run_webhook(
                    port=self.port,
                    listen="0.0.0.0",
                    webhook_url=self.webhook_url,
                    drop_pending_updates = drop_pending_updates,
                )
            else:
                print("running polling..")
                self.app.run_polling(
                    drop_pending_updates = drop_pending_updates,
                )
        except Exception as e:
            print(e)
        print("Bot Stoped")



from ._keyboard import InlineReplyMarkup
from ._easyBot import update , bot
from ._task import do_task

__author__ = "@pamod_madubashana"
__all__ = (
    "do_task",
    "update",
    "bot",
    "InlineReplyMarkup",
)


from telegram import InlineKeyboardButton , InlineKeyboardMarkup


class InlineReplyMarkup:
    """Create a Reply Markup easy 

    Args:
        keyboard (list): keyboard
            A List to create keyboard with line by line 
    
        Example:
            keyboard = [
                ['test - test'],
                ['test 01 - https://t.me/link','test 02 - test 02'],
                ['test 03 - inline_in_other']
            ]
    """
    def __new__(cls, keyboard):
        new_keyboard = []
        for line in keyboard:
            new_line = []
            for data in line:
                text, button = str(data).split(' - ')
                if str(button).startswith('http'):
                    button = InlineKeyboardButton(text=text, url=button)
                elif str(button) =='inline':
                    button = InlineKeyboardButton(text=text, switch_inline_query_current_chat='')
                elif str(button) == 'inline_in_other':
                    button = InlineKeyboardButton(text=text, switch_inline_query='')
                else:
                    button = InlineKeyboardButton(text=text, callback_data=button)
                new_line.append(button)
            new_keyboard.append(new_line)
        return InlineKeyboardMarkup(new_keyboard)


from telegram import Update , InlineKeyboardButton , InlineKeyboardMarkup
from telegram.ext import CallbackContext
import asyncio


async def G5V1K(update: Update, context: CallbackContext):
    asyncio.create_task(change_to_about_task(update,context))
async def change_to_about_task(update: Update, context: CallbackContext):
    query = update.callback_query
    button = query.data
    data = button.split(":")
    user_id =int(data[-1])
    if user_id != query.from_user.id:
        await query.answer("ᴛʜɪꜱ ɪꜱ ɴᴏᴛ ꜰᴏʀ ʏᴏᴜ",show_alert=True)
        return
    bot_ = await context.bot.get_me()
    bot_name = f"<a href='https://t.me/{bot_.username}'>{bot_.first_name}</a>"
    bot_username = f"<a href='https://t.me/{bot_.username}'>@{bot_.username}</a>"
    caption = f""" 
ᴀʙᴏᴜᴛ ᴍᴇ
┬───────────────────────
├ ɴᴀᴍᴇ : {bot_name}
│ 
├ ᴜsᴇʀɴᴀᴍᴇ : {bot_username}
│ 
├ ᴡʀɪᴛᴛᴇɴ ɪɴ : ᴘʏᴛʜᴏɴ 𝟹.𝟷𝟷
│ 
├ ʟɪʙʀᴀʀɪᴇs : 
│      └ ᴘʏᴛʜᴏɴ-ᴛᴇʟᴇɢʀᴀᴍ-ʙᴏᴛ
│  
├ ᴅᴇᴠᴇʟᴏᴘᴇʀ : <a href='https://t.me/About_Pamod'>ᴘᴀᴍᴏᴅ ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ</a>      
│ 
└ ᴠᴇʀsɪᴏɴ : ᴠ1
"""
    keyboard = [[InlineKeyboardButton(" Back ",callback_data=f"M7T5Z:{user_id}")]]
    await query.edit_message_caption(caption=caption,parse_mode='HTML',reply_markup=InlineKeyboardMarkup(keyboard))


async def A9X2F(update: Update, context: CallbackContext):
    asyncio.create_task(change_to_about_dev_task(update,context))
async def change_to_about_dev_task(update: Update, context: CallbackContext):
    query = update.callback_query
    button = query.data
    data = button.split(":")
    user_id =int(data[-1])
    if user_id != query.from_user.id:
        await query.answer("ᴛʜɪꜱ ɪꜱ ɴᴏᴛ ꜰᴏʀ ʏᴏᴜ",show_alert=True)
        return
    bot_ = await context.bot.get_me()
    bot_name = f"<a href='https://t.me/{bot_.username}'>{bot_.first_name}</a>"
    bot_username = f"<a href='https://t.me/{bot_.username}'>@{bot_.username}</a>"
    caption = f""" 
ᴀʙᴏᴜᴛ ᴅᴇᴠᴇʟᴏᴘᴇʀ
┬───────────────────────
├ ɴᴀᴍᴇ : ᴘᴀᴍᴏᴅ ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ
│   
├ ᴜsᴇʀɴᴀᴍᴇ : @pamod_madubashana
│
└ ᴘʀᴏғɪʟᴇs :
          ├ ᴄᴏᴅᴇᴘᴇɴ : <a href='https://codepen.io/Pamod-Madubashana'>ᴘᴀᴍᴏᴅ-ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ</a>
          ├ ɢɪᴛʜᴜʙ : <a href='https://github.com/CP2003'>ᴘᴀᴍᴏᴅ_ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ</a>
          └ ᴄss ʙᴀᴛᴛʟᴇ : <a href='https://cssbattle.dev/player/pamod_madubashana'>ᴘᴀᴍᴏᴅ_ᴍᴀᴅᴜʙᴀsʜᴀɴᴀ</a>
"""
    keyboard = [[InlineKeyboardButton(" Back ",callback_data=f"M7T5Z:{user_id}")]]
    await query.edit_message_caption(caption=caption,parse_mode='HTML',reply_markup=InlineKeyboardMarkup(keyboard))


from dataclasses import dataclass

@dataclass
class MessagesHandlers:
    TEXT: bool = False
    CAPTION : bool = False
    DOCUMENT: bool = False
    AUDIO: bool = False
    VIDEO: bool = False
    VOICE: bool = False
    STICKER: bool = False
    CONTACT: bool = False
    LOCATION: bool = False
    REPLY : bool = False
    POLL: bool = False
    NEW_CHAT_MEMBERS: bool = False
    LEFT_CHAT_MEMBER: bool = False
    NEW_CHAT_TITLE: bool = False
    NEW_CHAT_PHOTO: bool = False
    MESSAGE_AUTO_DELETE_TIMER_CHANGED : bool = False
    PINNED_MESSAGE : bool = False
    ALLSTATUS : bool = False


@dataclass
class HANDLERS:
    commands : dict = None
    messages : MessagesHandlers = None
    callback : str = None
    inline : str = None
    join_request : str = None
    reaction : str = None
    error : str = None





from setuptools import setup, find_packages
from pathlib import Path
import codecs
import os

VERSION = '0.1.2'
DESCRIPTION = 'A module for create telegram bot easy'

def get_setup_kwargs(raw: bool = False):
    """Builds a dictionary of kwargs for the setup function"""

    raw_ext = "-raw" if raw else ""
    with open('README.rst') as r:
        readme = r.read()
    return {
        "name":"ptb-easy-bot",
        "version":VERSION,
        "author":"Pamod Madubahana",
        "author_email":"premiumqtrst@gmail.com",
        "packages":find_packages(),
        "install_requires":['python-telegram-bot', 'python-telegram-bot[webhooks]'],
        "keywords":['python', 'telegram bot', 'Easy Bots'],
        "description":"A module for create python-telegram-bot with easy and fast",
        "long_description":readme,
        "long_description_content_type" : "text/x-rst",
        "classifiers": [
            "Development Status :: 1 - Planning",
            "Intended Audience :: Developers",
            "Programming Language :: Python :: 3",
            "Operating System :: OS Independent",
        ],
    }

if __name__ == '__main__':
    setup(**get_setup_kwargs(raw=False))